import logging
from typing import Optional

from core.user.schema.user import UserRole
from tech.handlers.animal import AnimalHandler
from tech.handlers.auth import AuthHandler
from tech.handlers.input import InputHandler
from tech.handlers.show import ShowHandler
from tech.handlers.user import UserHandler
from tech.utils.lang.langmodel import LanguageModel
from tech.utils.types import Menus, ConsoleMessage, UserConsoleInfo


class ConsoleHandler:
    user: Optional[UserConsoleInfo]
    animal_handler: AnimalHandler
    user_handler: UserHandler
    show_handler: ShowHandler
    auth_handler: AuthHandler
    input_handler: InputHandler
    lang_model: LanguageModel

    def __init__(self,
                 animal_handler: AnimalHandler,
                 show_handler: ShowHandler,
                 auth_handler: AuthHandler,
                 user_handler: UserHandler,
                 input_handler: InputHandler):
        self.animal_handler = animal_handler
        self.user_handler = user_handler
        self.show_handler = show_handler
        self.auth_handler = auth_handler
        self.input_handler = input_handler
        self.lang_model = self.input_handler.lang_model
        self.user = None

    def check_token(self) -> bool:
        if not self.auth_handler.verify_token(self.user.auth_details.access_token):
            print(self.lang_model.auth_token_expired)
            return False
        return True

    def select_judge(self) -> Optional[int]:
        logging.info('console: start judge select')
        run = True
        while run:
            if not self.check_token():
                return None

            option = self.input_handler.ask_question(Menus.judge_menu.value)
            if option == '0':
                return 0
            elif option == '1':  #
                self.show_handler.get_shows_all()
                return 1
            elif option == '2':  #
                self.show_handler.get_show_result()
                return 1
            elif option == '3':
                self.auth_handler.logout(self.user.auth_details.access_token)
                self.user = None
                return 1
            elif option == '4':  #
                self.show_handler.get_animals_by_show()
                return 1
            elif option == '5':  #
                self.show_handler.score_animal(self.user.id.value)
                return 1
            else:
                print(ConsoleMessage.input_invalid.value)

    def select_admin(self) -> Optional[int]:
        logging.info('console: start admin select')
        run = True
        while run:
            if not self.check_token():
                return None

            option = self.input_handler.ask_question(Menus.admin_menu.value)
            if option == '0':
                return 0
            elif option == '1':  #
                self.show_handler.get_shows_all()
                return 1
            elif option == '2':  #
                self.show_handler.get_show_result()
                return 1
            elif option == '3':
                self.auth_handler.logout(self.user.auth_details.access_token)
                self.user = None
                return 1
            elif option == '4':  #
                self.show_handler.create_show()
                return 1
            elif option == '5':  #
                self.show_handler.start_show()
                return 1
            elif option == '6':  #
                self.show_handler.stop_show()
                return 1
            elif option == '7':  #
                self.show_handler.register_user()
                return 1
            elif option == '8':  #
                self.show_handler.unregister_user()
                return 1
            else:
                print(ConsoleMessage.input_invalid.value)

    def select_breeder(self) -> Optional[int]:
        logging.info('console: start breeder select')
        run = True
        while run:
            if not self.check_token():
                return None

            option = self.input_handler.ask_question(Menus.breeder_menu.value)
            if option == '0':
                return 0
            elif option == '1':  #
                self.show_handler.get_shows_all()
                return 1
            elif option == '2':  #
                self.show_handler.get_show_result()
                return 1
            elif option == '3':
                self.auth_handler.logout(self.user.auth_details.access_token)
                self.user = None
                return 1
            elif option == '4':  #
                self.animal_handler.get_animals_by_user_id(self.user.id.value)
                return 1
            elif option == '5':  #
                self.animal_handler.create_animal(self.user.id.value)
                return 1
            elif option == '6':  #
                self.animal_handler.delete_animal(self.user.id.value)
                return 1
            elif option == '7':  #
                self.show_handler.register_animal(self.user.id.value)
                return 1
            elif option == '8':  #
                self.show_handler.unregister_animal(self.user.id.value)
                return 1
            else:
                print(ConsoleMessage.input_invalid.value)

    def set_user(self, new_user: UserConsoleInfo) -> None:
        logging.info('console: set user')
        if self.user is None:
            self.user = new_user
            return
        self.user.id = new_user.id
        self.user.email = new_user.email
        self.user.role = new_user.role
        self.user.name = new_user.name
        self.user.auth_details = new_user.auth_details

    def unset_user(self) -> None:
        logging.info('console: unset user')
        self.user.id = None
        self.user.email = None
        self.user.role = None
        self.user.name = None
        self.user.auth_details = None

    def select_guest(self) -> int:
        logging.info('console: start guest select')
        run = True
        while run:
            option = self.input_handler.ask_question(Menus.guest_menu.value)
            if option == '0':
                return 0
            elif option == '1':  #
                self.show_handler.get_shows_all()
                return 1
            elif option == '2':  #
                self.show_handler.get_show_result()
                return 1
            elif option == '3':
                res_user = self.auth_handler.signin()
                if res_user is not None:
                    self.set_user(res_user)
                return 1
            else:
                print(ConsoleMessage.input_invalid.value)

    def run(self):
        logging.info('console: run')
        stop = False
        while not stop:
            code = 1
            if self.user is None:
                code = self.select_guest()
            elif self.user.role == UserRole.breeder:
                code = self.select_breeder()
            elif self.user.role == UserRole.judge:
                code = self.select_judge()
            elif self.user.role == UserRole.admin:
                code = self.select_admin()

            if code == 0:
                stop = True
        logging.info('console: quit')



from tech.utils.lang.langmodel import LanguageModel


class RuLanguageModel(LanguageModel):
    not_owner_error = 'Текущий пользователь не владеет выбранным животным'
    not_judge_error = 'Текущий пользователь не является судьей на выбранной выставке'
    show_result_status_error = 'Нельзя посмотреть результаты: выставка не завершена'
    no_empty_field = 'Ввод не может быть пустым'
    yes = 'Да'
    no = 'Нет'
    out_login = 'Логин (почта): '
    out_password = 'Пароль: '
    role_register_error = 'Пользователь должен быть судьей'
    already_registered_error = 'Запись на выставку уже существует (нельзя записаться дважды)'
    not_registered_error = 'Записи на выбранную выставку не было'
    duplicate_error = 'Джекпот 777: найдет дупликат (ОШИБКА)'
    foreign_keys_error = 'Ошибка создания (проверьте существование сущностей, на которые введены ссылки)'
    out_deleted_success = 'Запись успешно удалена'
    out_deleted_error = 'Запись не может быть удалена'
    out_deleted_animal_active_shows_error = 'Животное не может быть удалено (есть запись на запущенные выставки)'
    input_invalid = 'Некорректный ввод'
    question_year = '[Ввод даты рождения] Введите год: '
    show_start_success = 'Выставка успешно запущена'
    show_stop_success = 'Выставка успешно завершена'
    unregister_animal_success = 'Животное успешно отписано от выставки'
    register_animal_success = 'Животное успешно записано на выставку'
    unregister_user_success = 'Судья успешно отписан от выставки'
    register_user_success = 'Судья успешно записан на выставку'
    show_start_error_status = 'Выставка не может быть запущена: неподходящий статус'
    show_start_error_no_records_user = 'Выставка не может быть запущена: нет записанных судей'
    show_stop_error_status = 'Выставка не может быть завершена: неподходящий статус'
    show_register_status_error = 'Невозможна запись на выставку: она должна быть в состоянии \'создана\''
    show_unregister_status_error = 'Невозможна отписка от выставки: она должна быть в состоянии \'создана\''
    show_stop_error_not_all_users_scored = 'Выставка не может быть завершена: не все судьи завершили оценивание'
    show_start_error_no_records_animal = 'Выставка не может быть запущена: нет записанных животных'
    question_month = '[Ввод даты рождения] Введите месяц: '
    question_day = '[Ввод даты рождения] Введите день: '
    auth_token_expired = 'Время сессии истекло'
    get_empty_result = 'Нет результатов'
    out_question_name = 'Завершить ввод имени? '
    out_question_animal_id = 'Завершить ввод ID животного? '
    out_question_show_id = 'Завершить ввод ID выставки? '
    out_question_breed_id = 'Завершить ввод ID породы? '
    out_question_species_id = 'Завершить ввод ID вида? '
    out_question_standard_id = 'Завершить ввод ID стандарта? '
    out_question_show_class = 'Завершить ввод класса? '
    out_question_country = 'Завершить ввод страны? '
    out_question_user_id = 'Завершить ввод ID пользователя? '
    out_question_length = 'Завершить ввод длины? '
    out_question_weight = 'Завершить ввод веса? '
    out_question_height = 'Завершить ввод роста? '
    out_question_sex = 'Завершить ввод пола? '
    out_question_has_defects = 'Завершить ввод признака дефектов? '
    out_question_is_multicolor = 'Завершить ввод признака многоцветности? '
    out_question_is_multi_breed = 'Завершить ввод признака мультипородности? '
    question_name = 'Введите имя: '
    question_breed_id = 'Введите ID породы: '
    question_show_id = 'Введите ID выставки: '
    question_animal_id = 'Введите ID животного: '
    question_species_id = 'Введите ID вида: '
    question_standard_id = 'Введите ID стандарта: '
    question_country = 'Введите страну (англ.): '
    question_show_class = 'Введите класс: '
    question_user_id = 'Введите ID пользователя: '
    question_length = 'Введите длину (см): '
    question_weight = 'Введите вес (кг): '
    question_height = 'Введите высоту (см): '
    question_status = 'Введите статус: '
    question_sex = 'Введите пол: '
    question_has_defects = 'Введите признак наличия дефектов (да/нет): '
    question_is_multicolor = 'Введите признак многоцветности (да/нет): '
    question_is_multi_breed = 'Является ли выставка мультипородной? (да/нет): '
    cancel_input = 'Ввод отменен'
    out_id = 'ID'
    out_rank = 'Место'
    question_score_value = 'Введите оценку (1-5): '
    out_question_score_value = 'Завершить ввод оценки? '
    question_animalshow_id = 'Введите ID записи животного на выставку: '
    question_usershow_id = 'Введите ID записи судьи на выставку: '
    out_question_animalshow_id = 'Завершить ввод ID записи животного на выставку? '
    out_question_usershow_id = 'Завершить ввод ID записи судьи на выставку? '
    out_animal_id = 'ID животного'
    out_show_id = 'ID выставки'
    out_breed_id = 'ID породы'
    out_species_id = 'ID вида'
    out_animalshow_id = 'ID записи судьи на выставку: '
    out_usershow_id = 'ID записи животного на выставку: '
    out_standard_id = 'ID стандарта'
    out_name = 'Имя'
    out_is_archived = 'В архиве'
    out_dt_created = 'Дата создания'
    score_create_success = 'Оценка успешно создана'
    user_not_found = 'Пользователь не найден'
    invalid_password = 'Неправильный пароль'
    animal_standard_error = 'Животное не соответствует стандарту выставки'
    animal_not_found = 'Животное не найдено'
    show_not_found = 'Выставка не найдена'
    out_score_value = 'Значение оценки'
    out_show_name = 'Название'
    out_birth_dt = 'Дата рождения'
    out_question_birth_dt = 'Завершить ввод даты рождения? '
    out_user_id = 'ID пользователя'
    out_length = 'Длина'
    out_show_class = 'Класс'
    out_status = 'Статус'
    out_country = 'Страна'
    out_weight = 'Вес'
    out_height = 'Рост'
    out_length_unit = 'см'
    out_weight_unit = 'кг'
    out_height_unit = 'см'
    out_has_defects = 'Наличие дефектов'
    out_is_multicolor = 'Многоцветность'
    out_is_multi_breed = 'Мультипородность'
    out_sex = 'Пол'
    out_sex_female = 'женский'
    out_sex_male = 'мужской'
    out_sex_female_short = 'ж'
    out_sex_male_short = 'м'



import logging

from core.animal.service.animal import IAnimalService
from core.show.schema.show import ShowSchemaReport
from core.show.service.animalshow import IAnimalShowService
from core.show.service.score import IScoreService
from core.show.service.show import IShowService
from core.show.service.usershow import IUserShowService
from core.utils.exceptions import NotFoundRepoError, StartShowStatusError, StartShowZeroRecordsError, \
    StopShowStatusError, StopNotAllUsersScoredError, ShowServiceError, UserShowServiceError, RegisterShowStatusError, \
    RegisterAnimalRegisteredError, RegisterUserRoleError, RegisterUserRegisteredError, UnregisterShowStatusError, \
    UnregisterUserNotRegisteredError, UnregisterAnimalNotRegisteredError, RegisterAnimalCheckError, \
    AnimalShowServiceError, ScoreServiceError
from core.utils.types import ID
from tech.dto.animal import AnimalDTO
from tech.dto.score import ScoreDTO
from tech.dto.show import ShowDTO
from tech.handlers.input import InputHandler
from tech.utils.exceptions import InputException
from tech.utils.lang.langmodel import LanguageModel
from utils.exceptions import ValidationRepoError


class ShowHandler:
    lm: LanguageModel
    animal_service: IAnimalService
    show_service: IShowService
    usershow_service: IUserShowService
    score_service: IScoreService
    input_handler: InputHandler
    animalshow_service: IAnimalShowService

    def __init__(self,
                 show_service: IShowService,
                 usershow_service: IUserShowService,
                 score_service: IScoreService,
                 input_handler: InputHandler,
                 animalshow_service: IAnimalShowService,
                 animal_service: IAnimalService):
        self.show_service = show_service
        self.usershow_service = usershow_service
        self.score_service = score_service
        self.input_handler = input_handler
        self.animalshow_service = animalshow_service
        self.lm = input_handler.lang_model
        self.animal_service = animal_service

    def score_animal(self, user_id: int):
        logging.info('show handler: score animal')
        show_id = self.input_handler.wait_positive_int(self.lm.question_show_id, self.lm.out_question_show_id)
        if show_id is None:
            return
        try:
            usershow_record = self.usershow_service.get_by_user_show_id(ID(user_id), ID(show_id))
        except UserShowServiceError:
            logging.warning('show handler: usershow duplicate error')
            print(self.lm.duplicate_error)
            return
        except NotFoundRepoError:
            logging.warning(f'show handler: user role error (not judge)')
            print(self.lm.not_judge_error)
            return

        animal_id = self.input_handler.wait_positive_int(self.lm.question_animal_id, self.lm.out_question_animal_id)
        if animal_id is None:
            return

        try:
            animalshow_record = self.animalshow_service.get_by_animal_show_id(ID(animal_id), ID(show_id))
        except AnimalShowServiceError:
            logging.warning('show handler: animalshow duplicate error')
            print(self.lm.duplicate_error)
            return
        except NotFoundRepoError:
            logging.warning(f'show handler: user role error (not judge)')
            print(self.lm.not_judge_error)
            return

        try:
            dto = ScoreDTO(input_handler=self.input_handler).input_create(usershow_record.id.value,
                                                                          animalshow_record.id.value)
        except InputException:
            logging.warning('show handler: invalid score input')
            return
        self.score_service.create(dto.to_schema_create())
        print(self.lm.score_create_success)

    def create_show(self):
        logging.info('show handler: create show')
        try:
            dto: ShowDTO = ShowDTO(input_handler=self.input_handler).input_create()
        except InputException:
            logging.warning('show handler: invalid create show input')
            return
        
        try:
            created = self.show_service.create(dto.to_schema_create())
        except ValidationRepoError:
            logging.warning(f'show handler: create show: repo validation error')
            print(self.lm.foreign_keys_error)
            return
        
        ShowDTO.from_schema(created, self.input_handler).print()

    def start_show(self):
        logging.info('show handler: start show')
        show_id = self.input_handler.wait_positive_int(self.lm.question_show_id, self.lm.out_question_show_id)
        if show_id is None:
            return
        try:
            self.show_service.start(ID(show_id))
        except StartShowStatusError:
            logging.warning('show handler: wrong start show status')
            print(self.lm.show_start_error_status)
            return
        except StartShowZeroRecordsError as e:
            if e.type == 'user':
                logging.warning('show handler: zero usershow records, cannot start')
                print(self.lm.show_start_error_no_records_user)
            elif e.type == 'animal':
                logging.warning('show handler: zero animalshow records, cannot start')
                print(self.lm.show_start_error_no_records_animal)
            return
        print(self.lm.show_start_success)

    def stop_show(self):
        logging.info('show handler: stop show')
        show_id = self.input_handler.wait_positive_int(self.lm.question_show_id, self.lm.out_question_show_id)
        if show_id is None:
            return
        try:
            self.show_service.stop(ID(show_id))
        except StopShowStatusError:
            logging.warning('show handler: wrong stop show status')
            print(self.lm.show_stop_error_status)
            return
        except StopNotAllUsersScoredError:
            logging.warning('show handler: not all users scored, cannot stop')
            print(self.lm.show_stop_error_not_all_users_scored)
            return
        except ScoreServiceError:
            print(self.lm.show_stop_error_not_all_users_scored)
            return
        print(self.lm.show_stop_success)

    def register_animal(self, user_id: int):
        logging.info('show handler: register animal')
        show_id = self.input_handler.wait_positive_int(self.lm.question_show_id, self.lm.out_question_show_id)
        if show_id is None:
            return

        animal_id = self.input_handler.wait_positive_int(self.lm.question_animal_id, self.lm.out_question_animal_id)
        if animal_id is None:
            return

        animal_dto = AnimalDTO.from_schema(self.animal_service.get_by_id(ID(animal_id)), self.input_handler)
        if animal_dto.user_id != user_id:
            print(self.lm.not_owner_error)
            return

        try:
            self.show_service.register_animal(ID(animal_id), ID(show_id))
        except RegisterShowStatusError:
            print(self.lm.show_register_status_error)
            return
        except RegisterAnimalRegisteredError:
            print(self.lm.already_registered_error)
            return
        except RegisterAnimalCheckError:
            print(self.lm.animal_standard_error)
            return

        print(self.lm.register_animal_success)

    def unregister_animal(self, user_id: int):
        logging.info('show handler: unregister animal')
        show_id = self.input_handler.wait_positive_int(self.lm.question_show_id, self.lm.out_question_show_id)
        if show_id is None:
            return

        animal_id = self.input_handler.wait_positive_int(self.lm.question_animal_id, self.lm.out_question_animal_id)
        if animal_id is None:
            return

        animal_dto = AnimalDTO.from_schema(self.animal_service.get_by_id(ID(animal_id)), self.input_handler)
        if animal_dto.user_id != user_id:
            print(self.lm.not_owner_error)
            return

        try:
            self.show_service.unregister_animal(ID(animal_id), ID(show_id))
        except UnregisterShowStatusError:
            print(self.lm.show_unregister_status_error)
            return
        except UnregisterAnimalNotRegisteredError:
            print(self.lm.not_registered_error)
            return
        print(self.lm.unregister_animal_success)

    def register_user(self):
        logging.info('show handler: register user')
        show_id = self.input_handler.wait_positive_int(self.lm.question_show_id, self.lm.out_question_show_id)
        if show_id is None:
            return
        user_id = self.input_handler.wait_positive_int(self.lm.question_user_id, self.lm.out_question_user_id)
        if user_id is None:
            return
        try:
            self.show_service.register_user(ID(user_id), ID(show_id))
        except RegisterShowStatusError:
            print(self.lm.show_register_status_error)
            return
        except RegisterUserRoleError:
            print(self.lm.role_register_error)
            return
        except RegisterUserRegisteredError:
            print(self.lm.already_registered_error)
            return
        print(self.lm.register_user_success)

    def unregister_user(self):
        logging.info('show handler: unregister user')
        show_id = self.input_handler.wait_positive_int(self.lm.question_show_id, self.lm.out_question_show_id)
        if show_id is None:
            return
        user_id = self.input_handler.wait_positive_int(self.lm.question_user_id, self.lm.out_question_user_id)
        if user_id is None:
            return
        try:
            self.show_service.unregister_user(ID(user_id), ID(show_id))
        except UnregisterShowStatusError:
            print(self.lm.show_unregister_status_error)
            return
        except UnregisterUserNotRegisteredError:
            print(self.lm.not_registered_error)
            return
        print(self.lm.unregister_user_success)

    def get_shows_all(self):
        logging.info('show handler: get all shows')
        res = self.show_service.get_all()
        if len(res) == 0:
            print(self.lm.get_empty_result)
            return
        for show in res:
            ShowDTO.from_schema(show, self.input_handler).print()

    def get_show_result(self):
        logging.info('show handler: get show result')
        show_id = self.input_handler.wait_positive_int(self.lm.question_show_id, self.lm.out_question_show_id)
        if show_id is None:
            return
        try:
            res: ShowSchemaReport = self.show_service.get_result_by_id(ID(show_id))
        except ShowServiceError:
            print(self.lm.show_result_status_error)
            return
        print('\n------РЕЗУЛЬТАТЫ------')  # TODO + score avg
        print(f'[{self.lm.out_rank} : {self.lm.out_animal_id}]')
        for rank_info in res.ranking_info:
            cur_animal_id = self.animalshow_service.get_by_id(rank_info.total_info.record_id).animal_id.value
            print(f'{self.lm.out_rank} {rank_info.rank}: {cur_animal_id}')
        print('----------------------')

    def get_animals_by_show(self):
        logging.info('show handler: get animals by show')
        show_id = self.input_handler.wait_positive_int(self.lm.question_show_id, self.lm.out_question_show_id)
        if show_id is None:
            return
        try:
            res = self.show_service.get_by_id_detailed(ID(show_id))
        except NotFoundRepoError:
            print(self.lm.get_empty_result)
            return
        if len(res.animals) == 0:
            print(self.lm.get_empty_result)
            return
        for animal in res.animals:
            AnimalDTO.from_schema(animal, self.input_handler).print()
    


import logging

from tech.handlers.input import InputHandler
from tech.utils.lang.langmodel import LanguageModel
from core.animal.service.animal import IAnimalService
from core.show.service.show import IShowService

from tech.dto.animal import AnimalDTO
from tech.utils.exceptions import InputException
from core.utils.exceptions import DeleteAnimalStartedShowError, NotFoundRepoError, ValidationRepoError
from core.utils.types import ID


class AnimalHandler:
    animal_service: IAnimalService
    show_service: IShowService
    input_handler: InputHandler
    lm: LanguageModel

    def __init__(self,
                 animal_service: IAnimalService,
                 show_service: IShowService,
                 input_handler: InputHandler):
        self.animal_service = animal_service
        self.show_service = show_service
        self.input_handler = input_handler
        self.lm = self.input_handler.lang_model

    def get_animals_by_user_id(self, user_id: int) -> None:
        logging.info(f'animal handler: get animals by user_id, user_id={user_id}')
        try:
            res = self.animal_service.get_by_user_id(ID(user_id))
        except NotFoundRepoError:
            logging.warning(f'animal handler: empty output animals by user_id={user_id}')
            print(self.lm.get_empty_result)
            return
        for animal in res:
            AnimalDTO.from_schema(animal, self.input_handler).print()

    def delete_animal(self, user_id: int) -> None:
        logging.info(f'animal handler: delete animal, user_id={user_id}')
        try:
            dto: AnimalDTO = AnimalDTO(input_handler=self.input_handler).input_delete()
        except InputException:
            logging.warning(f'animal handler: delete animal: invalid input')
            return

        existing_dto = AnimalDTO.from_schema(self.animal_service.get_by_id(ID(dto.id)), input_handler=self.input_handler)
        if existing_dto.user_id != user_id:
            logging.warning(f'animal handler: user isn\'t animal\'s owner, user_id={user_id} animal.user_id={existing_dto.user_id}')
            print(self.lm.not_owner_error)
            return

        try:
            res = self.animal_service.delete(ID(dto.id))
        except DeleteAnimalStartedShowError:
            logging.warning(f'animal handler: delete animal error: wrong show_status')
            print(self.lm.out_deleted_animal_active_shows_error)
            return
        print(self.lm.out_deleted_success + f' (ID: {res.id.value}, статус: {res.status})')

    def create_animal(self, user_id: int) -> None:
        logging.info(f'animal handler: create animal, user_id={user_id}')
        try:
            dto: AnimalDTO = AnimalDTO(input_handler=self.input_handler).input_create(user_id)
        except InputException:
            logging.warning(f'animal handler: create animal: invalid input')
            return
        try:
            created = self.animal_service.create(dto.to_schema_create())
        except ValidationRepoError:
            logging.warning(f'animal handler: create animal: repo validation error')
            print(self.lm.foreign_keys_error)
            return
        AnimalDTO.from_schema(created, self.input_handler).print()



import datetime
from typing import Optional

from core.animal.schema.animal import AnimalSchemaCreate, AnimalSchema
from core.utils.types import Sex, ID, AnimalName, Datetime, Weight, Height, Length

from tech.handlers.input import InputHandler
from tech.utils.exceptions import CancelInput, InvalidFloatInput, InvalidSexInput, InvalidBooleanInput
from tech.utils.lang.langmodel import LanguageModel


class AnimalDTO:
    id: int
    user_id: int
    breed_id: int
    name: str
    birth_dt: datetime.datetime
    sex: str
    weight: float
    height: float
    length: float
    has_defects: bool
    is_multicolor: bool
    input_handler: InputHandler
    lm: LanguageModel

    def __init__(self,
                 id: Optional[int] = None,
                 user_id: Optional[int] = None,
                 breed_id: Optional[int] = None,
                 name: Optional[str] = None,
                 birth_dt: Optional[datetime.datetime] = None,
                 sex: Optional[Sex] = None,
                 weight: Optional[float] = None,
                 height: Optional[float] = None,
                 length: Optional[float] = None,
                 has_defects: Optional[bool] = None,
                 is_multicolor: Optional[bool] = None,
                 input_handler: Optional[InputHandler] = None):
        if input_handler is not None:
            self.input_handler = input_handler
            self.lm = self.input_handler.lang_model
        if id is not None: self.id = id
        if user_id is not None: self.user_id = user_id
        if breed_id is not None: self.breed_id = breed_id
        if birth_dt is not None: self.birth_dt = birth_dt
        if sex is not None: self.sex = self.sex_from_enum(sex)
        if weight is not None: self.weight = weight
        if height is not None: self.height = height
        if length is not None: self.length = length
        if has_defects is not None: self.has_defects = has_defects
        if is_multicolor is not None: self.is_multicolor = is_multicolor
        if name is not None: self.name = name

    def sex_from_enum(self, sex: Sex):
        return self.lm.out_sex_female.lower() if sex == Sex.female else self.lm.out_sex_male.lower()

    def print(self):
        print("_______________________")
        print(f"{self.lm.out_id}:  {self.id}")
        print(f"{self.lm.out_name}: {self.name}")
        print(f"{self.lm.out_breed_id}: {self.breed_id}")
        print(f"{self.lm.out_user_id}: {self.user_id}")
        print(f"{self.lm.out_birth_dt}: {str(self.birth_dt)}")
        print(f"{self.lm.out_sex}: {self.sex}")
        print(f"{self.lm.out_weight}: {self.weight:5.3f} {self.lm.out_weight_unit}")
        print(f"{self.lm.out_height}: {self.height:5.1f} {self.lm.out_height_unit}")
        print(f"{self.lm.out_length}: {self.length:5.1f} {self.lm.out_length_unit}")
        print(f"{self.lm.out_has_defects}: {self.lm.yes if self.has_defects else self.lm.no}")
        print(f"{self.lm.out_is_multicolor}: {self.lm.yes if self.is_multicolor else self.lm.no}")

    def input_id(self):
        id = self.input_handler.wait_positive_int(
            self.lm.question_animal_id,
            self.lm.out_animal_id
        )
        if id is None:
            print(self.lm.cancel_input)
            raise CancelInput('animal id input cancel')
        self.id = id

    def input_delete(self):
        self.input_id()
        return self

    def input_create(self, user_id: int):
        self.id = 0
        self.input_name()
        self.input_breed_id()
        self.input_sex()
        self.input_birth_dt()
        self.input_weight()
        self.input_height()
        self.input_length()
        self.input_has_defects()
        self.input_is_multicolor()
        self.with_user_id(user_id)
        return self

    def input_birth_dt(self):
        birth_dt = self.input_handler.date_input(
            self.lm.out_question_birth_dt
        )
        if birth_dt is None:
            print(self.lm.cancel_input)
            raise CancelInput('animal birth_dt input cancel')
        self.birth_dt = birth_dt

    def with_user_id(self, user_id: int):
        self.user_id = user_id

    def input_breed_id(self):
        breed_id = self.input_handler.wait_positive_int(
            self.lm.question_breed_id,
            self.lm.out_breed_id
        )
        if breed_id is None:
            print(self.lm.cancel_input)
            raise CancelInput('animal breed_id input cancel')
        self.breed_id = breed_id

    def input_name(self):
        name = self.input_handler.wait_input(
            self.lm.question_name,
            self.lm.out_name
        )
        if name is None:
            print(self.lm.cancel_input)
            raise CancelInput('animal name input cancel')
        self.name = name

    def input_weight(self):
        weight = self.input_handler.wait_input(
            self.lm.question_weight,
            self.lm.out_weight
        )
        if weight is None:
            print(self.lm.cancel_input)
            raise CancelInput('animal weight input cancel')
        try:
            self.weight = float(weight)
        except ValueError:
            print(self.lm.input_invalid + f'({self.lm.out_weight.lower()})')
            raise InvalidFloatInput('weight')

    def input_length(self):
        length = self.input_handler.wait_input(
            self.lm.question_length,
            self.lm.out_length
        )
        if length is None:
            print(self.lm.cancel_input)
            raise CancelInput('animal length input cancel')
        try:
            self.length = float(length)
        except ValueError:
            print(self.lm.input_invalid + f'({self.lm.out_length.lower()})')
            raise InvalidFloatInput('length')

    def input_height(self):
        height = self.input_handler.wait_input(
            self.lm.question_height,
            self.lm.out_height
        )
        if height is None:
            print(self.lm.cancel_input)
            raise CancelInput('animal height input cancel')
        try:
            self.height = float(height)
        except ValueError:
            print(self.lm.input_invalid + f'({self.lm.out_height.lower()})')
            raise InvalidFloatInput('height')

    def input_sex(self):
        sex = self.input_handler.wait_input(
            self.lm.question_sex,
            self.lm.out_sex
        )
        if sex is None:
            print(self.lm.cancel_input)
            raise CancelInput('animal sex input cancel')
        if not self.sex_is_male(sex) and not self.sex_is_female(sex):
            print(self.lm.input_invalid + f'({self.lm.out_sex.lower()})')
            raise InvalidSexInput()
        self.sex = sex

    def sex_is_female(self, sex: str):
        return sex.upper() == self.lm.out_sex_female.upper() or sex.upper() == self.lm.out_sex_female_short.upper()

    def sex_is_male(self, sex: str):
        return sex.upper() == self.lm.out_sex_male.upper() or sex.upper() == self.lm.out_sex_male_short.upper()

    def input_has_defects(self):
        has_defects = self.input_handler.wait_input(self.lm.question_has_defects,
                                                    self.lm.out_question_has_defects)
        if has_defects is None:
            print(self.lm.cancel_input)
            raise CancelInput('animal has_defects input cancel')
        if has_defects.upper() != self.lm.yes.upper() and has_defects.upper() != self.lm.no.upper():
            print(self.lm.input_invalid + f'({self.lm.out_has_defects.lower()})')
            raise InvalidBooleanInput('has_defects')
        self.has_defects = True if has_defects.upper() == self.lm.yes.upper() else False

    def input_is_multicolor(self):
        is_multicolor = self.input_handler.wait_input(self.lm.question_is_multicolor,
                                                      self.lm.out_question_is_multicolor)
        if is_multicolor is None:
            print(self.lm.cancel_input)
            raise CancelInput('animal is_multicolor input cancel')
        if is_multicolor.upper() != self.lm.yes.upper() and is_multicolor.upper() != self.lm.no.upper():
            print(self.lm.input_invalid + f'({self.lm.out_is_multicolor.lower()})')
            raise InvalidBooleanInput('is_multicolor')
        self.is_multicolor = True if is_multicolor.upper() == self.lm.yes.upper() else False

    def sex_to_enum(self, sex: str):
        return Sex.female if self.sex_is_female(sex) else Sex.male

    def to_schema_create(self) -> AnimalSchemaCreate:
        return AnimalSchemaCreate(
            user_id=ID(self.user_id),
            breed_id=ID(self.breed_id),
            name=AnimalName(self.name),
            birth_dt=Datetime(self.birth_dt),
            sex=self.sex_to_enum(self.sex),
            weight=Weight(self.weight),
            height=Height(self.height),
            length=Length(self.length),
            has_defects=self.has_defects,
            is_multicolor=self.is_multicolor
        )

    @classmethod
    def from_schema(cls, other: AnimalSchema, input_handler: InputHandler):
        return cls(
            id=other.id.value,
            user_id=other.user_id.value,
            breed_id=other.breed_id.value,
            name=other.name.value,
            birth_dt=other.birth_dt.value,
            sex=other.sex,
            weight=other.weight.value,
            height=other.height.value,
            length=other.length.value,
            has_defects=other.has_defects,
            is_multicolor=other.is_multicolor,
            input_handler=input_handler
        )



from typing import Optional

from pydantic import BaseModel

from tech.handlers.input import InputHandler
from tech.utils.exceptions import CancelInput, InvalidBooleanInput
from tech.utils.lang.langmodel import LanguageModel
from core.show.schema.show import ShowSchemaCreate, ShowSchema, ShowClass, ShowStatus
from core.utils.types import ID, ShowName, Country


class ShowDTO:
    id: int
    species_id: Optional[int] = None
    breed_id: Optional[int] = None
    status: str
    country: str
    show_class: str
    name: str
    standard_id: Optional[int] = None
    is_multi_breed: bool
    input_handler: InputHandler
    lm: LanguageModel

    def print(self):
        print("_______________________")
        print(f"{self.lm.out_id}:  {self.id}")
        print(f"{self.lm.out_show_name}: {self.name}")
        print(f"{self.lm.out_status}: {self.status}")
        print(f"{self.lm.out_show_class}: {self.show_class}")
        print(f"{self.lm.out_country}: {self.country}")
        print(f"{self.lm.out_is_multi_breed}: {self.lm.yes if self.is_multi_breed else self.lm.no}")
        print(f"{self.lm.out_breed_id}: {'-' if self.breed_id is None else self.breed_id}")
        print(f"{self.lm.out_standard_id}: {'-' if self.standard_id is None else self.standard_id}")
        print(f"{self.lm.out_species_id}: {'-' if self.species_id is None else self.species_id}")

    def __init__(self, input_handler: Optional[InputHandler] = None,
        id: Optional[int] = None,
        species_id: Optional[int] = None,
        breed_id: Optional[int] = None,
        status: Optional[str] = None,
        country: Optional[str] = None,
        show_class: Optional[str] = None,
        name: Optional[str] = None,
        standard_id: Optional[int] = None,
        is_multi_breed: Optional[bool] = None
    ):
        if input_handler is not None:
            self.input_handler = input_handler
            self.lm = self.input_handler.lang_model
        if id is not None: self.id = id
        if species_id is not None: self.species_id = species_id
        if breed_id is not None: self.breed_id = breed_id
        if status is not None: self.status = status
        if country is not None: self.country = country
        if show_class is not None: self.show_class = show_class
        if name is not None: self.name = name
        if standard_id is not None: self.standard_id = standard_id
        if is_multi_breed is not None: self.is_multi_breed = is_multi_breed


    def input_id(self):
        id = self.input_handler.wait_positive_int(
            self.lm.question_show_id,
            self.lm.out_show_id
        )
        if id is None:
            print(self.lm.cancel_input)
            raise CancelInput('show id input cancel')
        self.id = id

    def input_delete(self):
        self.input_id()
        return self

    def input_create(self):
        self.id = 0
        self.input_name()
        self.input_country()
        self.input_show_class()
        self.input_is_multi_breed()
        if self.is_multi_breed:
            self.input_species_id()
            self.standard_id = None
            self.breed_id = None
        else:
            self.input_breed_id()
            self.input_standard_id()
            self.species_id = None
        self.status = ShowStatus.created
        return self

    def input_breed_id(self):
        breed_id = self.input_handler.wait_positive_int(
            self.lm.question_breed_id,
            self.lm.out_breed_id
        )
        if breed_id is None:
            print(self.lm.cancel_input)
            raise CancelInput('show breed_id input cancel')
        self.breed_id = breed_id

    def input_species_id(self):
        species_id = self.input_handler.wait_positive_int(
            self.lm.question_species_id,
            self.lm.out_species_id
        )
        if species_id is None:
            print(self.lm.cancel_input)
            raise CancelInput('show species_id input cancel')
        self.species_id = species_id

    def input_standard_id(self):
        standard_id = self.input_handler.wait_positive_int(
            self.lm.question_standard_id,
            self.lm.out_standard_id
        )
        if standard_id is None:
            print(self.lm.cancel_input)
            raise CancelInput('show standard_id input cancel')
        self.standard_id = standard_id

    def input_name(self):
        name = self.input_handler.wait_input(
            self.lm.question_name,
            self.lm.out_name
        )
        if name is None:
            print(self.lm.cancel_input)
            raise CancelInput('show name input cancel')
        self.name = name

    def input_status(self):
        status = self.input_handler.wait_input(
            self.lm.question_status,
            self.lm.out_status
        )
        if status is None:
            print(self.lm.cancel_input)
            raise CancelInput('show status input cancel')
        self.status = status
        # TODO: check status

    def input_country(self):
        country = self.input_handler.wait_input(
            self.lm.question_country,
            self.lm.out_country
        )
        if country is None:
            print(self.lm.cancel_input)
            raise CancelInput('show country input cancel')
        self.country = country
        # TODO: check country

    def input_show_class(self):
        show_class = self.input_handler.wait_input(
            self.lm.question_show_class,
            self.lm.out_show_class
        )
        if show_class is None:
            print(self.lm.cancel_input)
            raise CancelInput('show show_class input cancel')
        self.show_class = show_class
        # TODO: chech class

    def input_is_multi_breed(self):
        is_multi_breed = self.input_handler.wait_input(self.lm.question_is_multi_breed,
                                                       self.lm.out_question_is_multi_breed)
        if is_multi_breed is None:
            print(self.lm.cancel_input)
            raise CancelInput('show is_multi_breed input cancel')
        if is_multi_breed.upper() != self.lm.yes.upper() and is_multi_breed.upper() != self.lm.no.upper():
            print(self.lm.cancel_input)
            raise InvalidBooleanInput('is_multi_breed')
        self.is_multi_breed = True if is_multi_breed.upper() == self.lm.yes.upper() else False

    def to_schema_create(self) -> ShowSchemaCreate:
        standard_id = self.standard_id if self.standard_id is None else ID(self.standard_id)
        species_id = self.species_id if self.species_id is None else ID(self.species_id)
        breed_id = self.breed_id if self.breed_id is None else ID(self.breed_id)
        return ShowSchemaCreate(
            species_id=species_id,
            breed_id=breed_id,
            country=Country(self.country),
            show_class=ShowClass(self.show_class),
            name=ShowName(self.name),
            standard_id=standard_id,
            is_multi_breed=self.is_multi_breed
        )

    @classmethod
    def from_schema(cls, other: ShowSchema, input_handler: InputHandler):
        standard_id = other.standard_id if other.standard_id is None else other.standard_id.value
        species_id = other.species_id if other.species_id is None else other.species_id.value
        breed_id = other.breed_id if other.breed_id is None else other.breed_id.value
        return cls(
            id=other.id.value,
            species_id=species_id,
            breed_id=breed_id,
            country=other.country.value,
            show_class=other.show_class.value,
            status=other.status.value,
            name=other.name.value,
            standard_id=standard_id,
            is_multi_breed=other.is_multi_breed,
            input_handler=input_handler
        )



import datetime
import logging
from typing import Optional

from core.auth.schema.auth import Token
from tech.utils.types import UserConsoleInfo
from tech.handlers.input import InputHandler
from tech.utils.lang.langmodel import LanguageModel
from auth_provider.utils.exceptions import AuthProviderError
from core.auth.schema.auth import AuthSchemaSignIn, AuthDetails
from core.auth.service.auth import IAuthService
from core.user.service.user import IUserService
from core.utils.exceptions import NotFoundRepoError, AuthServiceError, SignInNotFoundEmailError, SignInPasswordError
from core.utils.types import Email, Fingerprint


class AuthHandler:
    auth_service: IAuthService
    lm: LanguageModel
    input_handler: InputHandler
    user_service: IUserService

    def __init__(self, auth_service: IAuthService,
                 user_service: IUserService,
                 input_handler: InputHandler):
        self.input_handler = input_handler
        self.auth_service = auth_service
        self.user_service = user_service
        self.lm = input_handler.lang_model

    def signin(self) -> Optional[UserConsoleInfo]:
        logging.info('auth handler: sign in')
        while True:
            email = self.input_handler.ask_question(self.lm.out_login)

            try:
                Email(email)
            except ValueError:
                logging.warning('auth handler: sign in: email validation error')
                print(self.lm.input_invalid)
                return None

            password = self.input_handler.ask_question(self.lm.out_password)

            try:
                res_user = self.user_service.get_by_email(Email(email))
            except NotFoundRepoError:
                logging.warning(f'auth handler: sign in: no user with email={email}')
                print(self.lm.user_not_found)
                return None

            try:
                res: AuthDetails = self.auth_service.signin(AuthSchemaSignIn(email=Email(email), password=password,
                                                          fingerprint=Fingerprint(value=str(datetime.datetime.now()))))
            except SignInNotFoundEmailError:
                print(self.lm.user_not_found)
                logging.warning(f'auth handler: sign in: no user with email={email}')
                return None
            except SignInPasswordError:
                print(self.lm.invalid_password)
                logging.warning(f'auth handler: sign in: wrong password, email={email}')
                return None

            return UserConsoleInfo(
                id=res_user.id,
                email=res_user.email,
                role=res_user.role,
                name=res_user.name,
                auth_details=res
            )


    def verify_token(self, token: Token) -> bool:
        logging.info('auth handler: verify token')
        try:
            self.auth_service.verify_token(token)
        except AuthProviderError:
            logging.info('auth handler: wrong token')
            return False
        return True

    def logout(self, token: Token) -> None:
        logging.info('auth handler: logout')
        self.auth_service.logout(token)



from pydantic import BaseModel


class LanguageModel:
    not_owner_error: str
    not_judge_error: str
    animal_not_found: str
    show_not_found: str
    foreign_keys_error: str
    duplicate_error: str
    show_result_status_error: str
    question_status: str
    no_empty_field: str
    invalid_password: str
    animal_standard_error: str
    out_password: str
    yes: str
    no: str
    role_register_error: str
    show_unregister_status_error: str
    not_registered_error: str
    already_registered_error: str
    show_register_status_error: str
    user_not_found: str
    input_invalid: str
    question_year: str
    question_month: str
    question_day: str
    out_deleted_success: str
    out_deleted_error: str
    out_deleted_animal_active_shows_error: str
    auth_token_expired: str
    get_empty_result: str
    question_name: str
    out_question_name: str
    out_question_breed_id: str
    out_question_animal_id: str
    out_question_show_id: str
    show_start_success: str
    show_stop_success: str
    unregister_animal_success: str
    score_create_success: str
    register_animal_success: str
    out_login: str
    unregister_user_success: str
    register_user_success: str
    show_stop_error_status: str
    show_stop_error_not_all_users_scored: str
    show_start_error_status: str
    show_start_error_no_records_user: str
    show_start_error_no_records_animal: str
    out_question_birth_dt: str
    out_question_user_id: str
    out_question_length: str
    out_question_weight: str
    out_question_height: str
    out_question_sex: str
    out_question_has_defects: str
    out_question_is_multicolor: str
    out_question_is_multi_breed: str
    out_question_show_class: str
    out_question_country: str
    question_breed_id: str
    question_species_id: str
    question_standard_id: str
    out_question_species_id: str
    out_question_standard_id: str
    question_animal_id: str
    question_animalshow_id: str
    question_usershow_id: str
    out_question_animalshow_id: str
    out_question_usershow_id: str
    question_show_id: str
    question_birth_dt: str
    question_user_id: str
    question_length: str
    question_country: str
    question_show_class: str
    question_weight: str
    question_height: str
    question_sex: str
    question_has_defects: str
    question_is_multicolor: str
    question_is_multi_breed: str
    cancel_input: str
    out_id: str
    out_rank: str
    out_animal_id: str
    out_show_id: str
    out_animalshow_id: str
    out_usershow_id: str
    out_breed_id: str
    out_show_name: str
    out_name: str
    out_score_value: str
    out_is_archived: str
    out_dt_created: str
    out_birth_dt: str
    out_user_id: str
    out_length: str
    out_weight: str
    out_height: str
    out_length_unit: str
    out_weight_unit: str
    out_height_unit: str
    out_has_defects: str
    out_is_multicolor: str
    out_is_multi_breed: str
    out_species_id: str
    out_standard_id: str
    out_sex: str
    out_show_class: str
    out_status: str
    out_country: str
    out_sex_female: str
    out_sex_male: str
    out_sex_female_short: str
    out_sex_male_short: str
    question_score_value: str
    out_question_score_value: str



from typing import Any


class InputException(Exception):
    def __init__(self, detail: Any = None):
        super().__init__(detail)


class CancelInput(InputException):
    def __init__(self, detail: Any = None):
        super().__init__(detail)


class InvalidFloatInput(InputException):
    def __init__(self, detail: Any = None):
        super().__init__(detail)


class InvalidSexInput(InputException):
    def __init__(self, detail: Any = None):
        super().__init__(detail)


class InvalidScoreInput(InputException):
    def __init__(self, detail: Any = None):
        super().__init__(detail)


class InvalidBooleanInput(InputException):
    def __init__(self, detail: Any = None):
        super().__init__(detail)



import datetime
import logging
from typing import Optional

from pydantic import PositiveInt

from tech.utils.lang.langmodel import LanguageModel


class InputHandler:
    lang_model: LanguageModel

    def __init__(self, lang_model: LanguageModel):
        self.lang_model = lang_model

    @staticmethod
    def ask_question(query: str) -> str:
        print(query, end='')
        return input('')

    def wait_input(self, question: str, out_question: str) -> Optional[str]:
        logging.info('input handler: wait string input')
        input = True
        while input:
            res = self.ask_question(question)
            if res == '' or res == '\n':
                print(self.lang_model.no_empty_field)
            else:
                return res
            out = self.ask_question(out_question)
            if out == '' or out == '\n' or self.lang_model.yes.lower() == out.lower():
                input = False
        return None

    def wait_positive_int(self, question: str, out_question: str) -> Optional[PositiveInt]:
        logging.info('input handler: wait positive int')
        input = True
        while input:
            res = self.ask_question(question)
            try:
                PositiveInt(res)
            except ValueError:
                print(self.lang_model.input_invalid)
            else:
                return res
            out = self.ask_question(out_question)
            if out == '' or out == '\n' or self.lang_model.yes.lower() == out.lower():
                input = False
        return None

    def date_input(self, out_question: str) -> Optional[datetime.datetime]:
        logging.info('input handler: input date')
        input = True
        while input:
            day = self.ask_question(self.lang_model.question_day)
            month = self.ask_question(self.lang_model.question_month)
            year = self.ask_question(self.lang_model.question_year)

            if day == '' or day == '\n' or month == '' or month == '\n' or year == '' or year == '\n':
                print(self.lang_model.no_empty_field)
            else:
                try:
                    PositiveInt(day)
                    PositiveInt(month)
                    PositiveInt(year)
                except ValueError:
                    print(self.lang_model.input_invalid)
                else:
                    try:
                        date = datetime.datetime(PositiveInt(year), PositiveInt(month), PositiveInt(day))
                    except ValueError:
                        print(self.lang_model.input_invalid)
                    else:
                        return date
            out = self.ask_question(out_question)
            if out == '' or out == '\n' or self.lang_model.yes.lower() == out.lower():
                input = False
        return None



import enum
from dataclasses import dataclass
from typing import Optional

from pydantic import BaseModel

from core.auth.schema.auth import AuthDetails
from core.user.schema.user import UserRole
from core.utils.types import ID, Email, UserName


class UserConsoleInfo(BaseModel):
    id: Optional[ID]
    email: Optional[Email]
    role: Optional[UserRole]
    name: Optional[UserName]
    auth_details: Optional[AuthDetails]
    

@dataclass
class Option:
    value: int


@enum.unique
class ConsoleInputStatus(str, enum.Enum):
    ok = 'Корректный ввод'
    invalid = 'Некорректный ввод'


@dataclass
class ConsoleInputResponse:
    res: str
    status: ConsoleInputStatus


@enum.unique
class Menus(str, enum.Enum):
    guest_menu = '\nМеню (Гость):\n'\
                  '0. Завершить работу\n'\
                  '1. Посмотреть список выставок\n'\
                  '2. Посмотреть результаты выставки\n'\
                  '3. Войти\n'\
                  'Выберите пункт: '
    admin_menu = '\nМеню (Организатор):\n'\
                  '0. Завершить работу\n'\
                  '1. Посмотреть список выставок\n'\
                  '2. Посмотреть результаты выставки\n'\
                  '3. Выйти\n'\
                  '4. Создать выставку\n'\
                  '5. Запустить выставку\n'\
                  '6. Завершить выставку\n'\
                  '7. Добавить судью на выставку\n'\
                  '8. Удалить судью с выставки\n'\
                  'Выберите пункт: '
    breeder_menu = '\nМеню (Заводчик):\n'\
                    '0. Завершить работу\n'\
                    '1. Посмотреть список выставок\n'\
                    '2. Посмотреть результаты выставки\n'\
                    '3. Выйти\n'\
                    '4. Список животных\n'\
                    '5. Добавить животное\n'\
                    '6. Удалить животное\n'\
                    '7. Записать животное на выставку\n'\
                    '8. Отписать животное от выставки\n'\
                  'Выберите пункт: '
    judge_menu = '\nМеню (Судья):\n'\
                  '0. Завершить работу\n'\
                  '1. Посмотреть список выставок\n'\
                  '2. Посмотреть результаты выставки\n'\
                  '3. Выйти\n'\
                  '4. Посмотреть участников выставки\n'\
                  '5. Оценить участника выставки\n'\
                  'Выберите пункт: '


@enum.unique
class ConsoleMessage(str, enum.Enum):
    input_invalid: str = 'Ошибка выбора пункта'



import datetime
from typing import Optional

from tech.handlers.input import InputHandler
from tech.utils.exceptions import CancelInput, InvalidScoreInput
from tech.utils.lang.langmodel import LanguageModel
from core.show.schema.score import ScoreSchemaCreate
from core.utils.types import MAX_SCORE_VALUE, Datetime, ScoreValue, ID


class ScoreDTO:
    id: int
    usershow_id: int
    animalshow_id: int
    value: int
    is_archived: bool
    dt_created: datetime.datetime
    input_handler: InputHandler
    lm: LanguageModel

    def __init__(self,
                 id: Optional[int] = None,
                 usershow_id: Optional[int] = None,
                 animalshow_id: Optional[int] = None,
                 dt_created: Optional[datetime.datetime] = None,
                 value: Optional[int] = None,
                 is_archived: Optional[bool] = None,
                 input_handler: Optional[InputHandler] = None):
        if input_handler is not None:
            self.input_handler = input_handler
            self.lm = self.input_handler.lang_model
        if id is not None: self.id = id
        if usershow_id is not None: self.usershow_id = usershow_id
        if animalshow_id is not None: self.animalshow_id = animalshow_id
        if dt_created is not None: self.dt_created = dt_created
        if value is not None: self.value = value
        if is_archived is not None: self.is_archived = is_archived

    def input_id(self):
        id = self.input_handler.wait_positive_int(
            self.lm.question_show_id,
            self.lm.out_show_id
        )
        if id is None:
            print(self.lm.cancel_input)
            raise CancelInput('show id input cancel')
        self.id = id

    def input_animalshow_id(self):
        animalshow_id = self.input_handler.wait_positive_int(
            self.lm.question_animalshow_id,
            self.lm.out_animalshow_id
        )
        if animalshow_id is None:
            print(self.lm.cancel_input)
            raise CancelInput('score animalshow_id input cancel')
        self.animalshow_id = animalshow_id

    def input_usershow_id(self):
        usershow_id = self.input_handler.wait_positive_int(
            self.lm.question_usershow_id,
            self.lm.out_usershow_id
        )
        if usershow_id is None:
            print(self.lm.cancel_input)
            raise CancelInput('score usershow_id input cancel')
        self.usershow_id = usershow_id

    def input_value(self):
        value = self.input_handler.wait_positive_int(
            self.lm.question_score_value,
            self.lm.out_score_value
        )
        if value is None:
            print(self.lm.cancel_input)
            raise CancelInput('show value input cancel')
        int_value = int(value)
        if int_value > MAX_SCORE_VALUE or int_value < 1:
            print(self.lm.input_invalid)
            raise InvalidScoreInput('show value invalid input (must be from 1 to 5)')
        self.value = int_value

    def input_create(self, usershow_id: int, animalshow_id: int):
        self.usershow_id = usershow_id
        self.animalshow_id = animalshow_id
        self.input_value()
        self.is_archived = False
        self.dt_created = datetime.datetime.now()
        self.id = 0
        return self

    def print(self):
        print("_______________________")
        print(f"{self.lm.out_id}:  {self.id}")
        print(f"{self.lm.out_usershow_id}: {self.usershow_id}")
        print(f"{self.lm.out_animalshow_id}: {self.animalshow_id}")
        print(f"{self.lm.out_score_value}: {self.value}")
        print(f"{self.lm.out_dt_created}: {str(self.dt_created.date())}")
        print(f"{self.lm.out_is_archived}: {self.lm.yes if self.is_archived else self.lm.no}")

    def to_schema_create(self) -> ScoreSchemaCreate:
        return ScoreSchemaCreate(
            usershow_id=ID(self.usershow_id),
            animalshow_id=ID(self.animalshow_id),
            value=ScoreValue(self.value),
            dt_created=Datetime(self.dt_created)
        )



from setuptools import setup, find_packages
from tech import __version__

setup(
    classifiers=[
        'Programming Language :: Python :: 3.10',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
    ],
    name='petowo-tech',
    version=__version__,
    url='https://github.com/fleshbound/petowo-tech',
    author='Valeria Avdeykina',
    author_email='avdeykina@icloud.com',

    packages=find_packages(),

    install_requires=[
        'typing',
        'pydantic==2.7.1',
        'logging',
        'datetime',
        'petowo-core'
    ]
)
